from .agilent_lightwave_connection import AgilentLightWaveConnection
from .lasers.agilent_8164B_laser import LaserAgilent8164B
from .power_meters.agilent_8164B_power_meter import PowerMeterAgilent8164B

class AgilentLightWaveSystem(AgilentLightWaveConnection):
    '''
    Driver for the Agilent Lightwave.
    Args:
        gpib_num (int): The number of the GPIB bus
            the power meter is sitting on.
        gpib_dev_num (int): The device number that
            the power meter is on the aforementioned bus.
        power_meter_channel_num (int): Either `1` or `2`
            depending on which power metre channel to use.
        output_mode (str):
            'HIGH' -> The High Power output is regulated.
            'LOWS' -> The Low SSE output is regulated.
            'BHR' -> Both outputs are active but only the
                High Power output is Regulated.
            'BLR' -> Both outputs are active but only the
                Low SSE output is Regulated.
        power_unit (str): Either \'W\' or \'dBm\' depending
            on whether the power units should be displayed
            in [W] or [dBm] on the Agielent 8164B\'s screen.
    '''
    def __init__(self, gpib_num, gpib_dev_num):
        super().__init__(gpib_num, gpib_dev_num)
        if not self.get_lock_status():
            self.set_unlock()

    def get_lock_status(self):
        lock_status = bool(int(self._query('lock?')))
        return lock_status

    def set_lock(self, password='1234'):
        assert len(password) == 4, 'Password should be 4 characters long.'
        self._write('lock 1,%s' % password)
        return self.get_lock_status()

    def set_unlock(self, password='1234'):
        assert len(password) == 4, 'Password should be 4 characters long.'
        self._write('lock 0,%s' % password)
        return self.get_lock_status()

    def get_modules_installed(self):
        return self._query('*OPT?').strip().replace(' ', '').split(',')

    def get_latest_error(self):
        return self._query('syst:err?')

    def clear_error_list(self):
        self._write('*CLS')
        return self.get_latest_error()

    def set_preset(self):
        self._write('*RST')

class AgilentLightWave():
    def __init__(self, gpib_num, gpib_dev_num, pm_sensor_num, pm_channel_num=1,
                 laser_output_mode='high', power_unit='W'):

        self.system = AgilentLightWaveSystem(gpib_num, gpib_dev_num)

        self.laser = LaserAgilent8164B(gpib_num,
                                       gpib_dev_num,
                                       power_unit,
                                       laser_output_mode)
        self.power_meter = PowerMeterAgilent8164B(gpib_num,
                                                  gpib_dev_num,
                                                  pm_sensor_num,
                                                  pm_channel_num,
                                                  power_unit)




from . import power_meter as pm
from .agilent_lightwave_connection import AgilentLightWaveConnection

class PowerMeterAgilent8164B(AgilentLightWaveConnection, pm.PowerMeter):
    '''
    Driver for the Agilent 8164B power meter module that
    can read power and set the wavelength as well as
    specify in what units the Agilent 8164B should operate
    in.

    Args:
        gpib_num (int): The number of the GPIB bus
            the power meter is sitting on.
        gpib_dev_num (int): The device number that
            the power meter is on the aforementioned bus.
        channel_num (int): Either `1` or `2` depending on
            which power metre channel to use.
        power_unit (str): Either \'W\' or \'dBm\' depending
            on whether the power units should be displayed
            in [W] or [dBm] on the Agielent 8164B\'s screen.
    '''
    def __init__(self, gpib_num=None, gpib_dev_num=None, sensor_num=1, channel_num=1, power_unit='W'):
        super().__init__(gpib_num=gpib_num, gpib_dev_num=gpib_dev_num)
        self._sensor_num = int(sensor_num)
        self._channel_num = int(channel_num)

    def _get_power_W(self):
        instr = 'fetc' + str(self._sensor_num) + ':chan%i:pow?' % self._channel_num
        power = float(self._query(instr))
        if self._unit == 'dbm':
            power = pm.PowerMeter.dbm_to_watts(power)
        return power

    def set_wavelength_m(self, wavelength_m):
        cmd = 'sens' + str(self._sensor_num) + ':chan' + str(self._channel_num) + ':pow:wav ' + str(wavelength_m) + 'm'
        self._write(cmd)
        r = self.get_wavelength_m()
        return r

    def get_wavelength_m(self):
        cmd = 'sens' + str(self._sensor_num) + ':chan' + str(self._channel_num) + ':pow:wav?'
        wl_m_str = self._query(cmd)
        wl_m = float(wl_m_str)
        return wl_m

    def set_unit(self, unit):
        '''
        Sets the units the power meter will display on screen.

        Args:
            unit (str): Either \'W\' or \'dBm\' depending on
                whether the power units should be displayed in
                [W] or [dBm] on the Agielent 8164B\'s screen.

        Returns:
            str: The unit the power meter is set to display.
                Either \'W\' or \'dBm\'
        '''
        assert unit.lower() in ('dbm', 'w')
        cmd = 'sens' + str(self._sensor_num) + ':chan' + str(self._channel_num) + ':pow:unit ' + unit
        self._write(cmd)
        r = self.get_unit()
        self._unit = unit.lower()
        return r

    def get_unit(self):
        '''
        Gets the units the power meter is set to display.

        Returns:
            str: The unit the power meter is set to display.
                Either \'W\' or \'dBm\'
        '''
        cmd = 'sens' + str(self._sensor_num) + ':chan' + str(self._channel_num) + ':pow:unit?'
        data = self._query(cmd)
        unit =  int(data)
        if unit == 0:
            r = 'dBm'
        else:
            r = 'W'
        return r

    def get_averaging_time_s(self):
        cmd = 'sens' + str(self._sensor_num) + ':chan' + str(self._channel_num) + ':pow:atim?'
        data = self._query(cmd)
        return data

    def set_averaging_time_s(self, averaging_time_s):
        cmd = 'sens' + str(self._sensor_num) + ':chan' + str(self._channel_num) + ':pow:atim %fs' % averaging_time_s
        data = self._write(cmd)
        return self.get_averaging_time_s()

    def set_auto_range(self):
        '''
        Turns on auto-range.

        Returns:
            bool: `1` if auto-range is switched on and
                `0` if auto-range is switched off.
        '''
        self._write('sens' + str(self._sensor_num) + ':pow:range:auto 1')
        r = self.get_auto_range()
        return r

    def unset_auto_range(self):
        '''
        Turns off auto-range.

        Returns:
            bool: `1` if auto-range is switched on and
                `0` if auto-range is switched off.
        '''
        self._write('sens' + str(self._sensor_num) + ':pow:range:auto 0')
        r = self.get_auto_range()
        return r

    def set_range_dbm(self, range_dBm):
        self._write('sens' + str(self._sensor_num) + ':pow:rang %.2fdbm' % range_dbm)
        r = self.get_range_dbm()
        return r

    def get_range_dbm(self):
        r = self._query('sens' + str(self._sensor_num) + ':pow:rang?')
        r = float(r)
        assert -110. <= r <= 30.
        return r

    def get_power_meter_range(self):
        data = self.ask('sens' + str(self._sensor_num) + ':chan' + str(self._channel_num) + ':pow:range?')
        return float(data)

    def set_power_meter_range(self, max_power_dbm):
        """Set the power meter range, range in dBm"""
        self._write('sens' + str(self._sensor_num) + ':chan' + str(self._channel_num) + ':pow:range ' + \
                        str(max_power_dbm) + 'dbm')
        return self.get_power_meter_range()

    def get_auto_range(self):
        '''
        Checks what the auto-range is set to.

        Returns:
            bool: `1` if auto-range is switched on and
                `0` if auto-range is switched off.
        '''
        data = self._query('sens' + str(self._sensor_num) + ':pow:range:auto?')
        r = bool(int(data))
        return r

    def get_analogue_current_A(self, analogue_voltage_V):
        raise NotImplementedError()

    def get_responsivity_A_W(self):
        raise NotImplementedError()



import gpib
import serial as ser

class AgilentLightWaveConnection():
    def __init__(self, gpib_num, gpib_dev_num):
        self._dev = gpib.dev(gpib_num, gpib_dev_num)

    def _write(self, cmd):
        gpib.write(self._dev, cmd)

    def _read(self, num_bytes=100):
        data = gpib.read(self._dev, num_bytes)
        return data.decode('ascii')

    def _read_raw(self, num_bytes=100):
        data = gpib.read(self._dev, num_bytes)
        return data

    def _query(self, cmd, num_bytes=100):
        self._write(cmd)
        data = self._read(num_bytes)
        return data

    def _query_raw(self, cmd, num_bytes=100):
        self._write(cmd)
        data = self._read_raw(num_bytes)
        return data



import abc
import math
import numpy as np
from scipy import interpolate
from tqdm import tqdm
import time

class Laser(object, metaclass=abc.ABCMeta):
    '''
    Abstract base interface class for lasers.

    All laser drivers should derive from this class
    and call `super.__init__(...)` in their constructor.
    '''
    @abc.abstractmethod
    def turn_on(self):
        '''Turns the laser on.

        Returns:
            bool: `True` if the laser is on, `False` if the laser is off.
                (Should be `True`.)
        '''
        pass

    @abc.abstractmethod
    def turn_off(self):
        '''Turns the laser off.

        Returns:
            bool: `True` if the laser is off, `False` if the laser is on.
                (Should be `False`.)
        '''
        pass

    @abc.abstractmethod
    def get_on_or_off(self):
        '''Checks if the laser is on or off.

        Returns:
            bool: `True` if the laser is off, `False` if the laser is on.
        '''
        pass

    @abc.abstractmethod
    def set_power_W(self, power_W):
        '''Sets the power of the laser in [W].

        Args:
            power_W (int, float): power in [W] to set the laser to.

        Returns:
            float: The power of the laser.
        '''
        pass

    @abc.abstractmethod
    def get_power_W(self):
        '''Gets the power of the laser in [W].

        Returns:
            float: The power of the laser in [W].
        '''
        pass

    @staticmethod
    def dbm_to_watts(power_dbm):
        '''Converts [dBm] to [W].

        Args:
            power_dbm (int, float): Power in [dBm].

        Returns:
            float: Power in [W].
        '''
        power_W = 10.**(power_dbm/10.) / 1.e3
        return power_W

    @staticmethod
    def watts_to_dbm(power_W):
        '''Converts [W] to [dBm].

        Args:
            power_W (int, float): Power in [W].

        Returns:
            float: Power in [dBm].
        '''
        power_dbm = 10.*math.log10(power_W/1.e-3)
        return power_dbm

    def set_power_mW(self, power_mW):
        return self.set_power_W(power_mW * 1.e-3) * 1.e3

    def set_power_uW(self, power_uW):
        return self.set_power_W(power_uW * 1.e-6) * 1.e6

    def set_power_dbm(self, power_dbm):
        power_W = self.dbm_to_watts(power_dbm)
        return self.set_power_W(power_W)

    def get_power_mW(self):
        return self.get_power_W()* 1.e3

    def get_power_uW(self):
        return self.get_power_W() * 1.e6

    def get_power_nW(self):
        return self.get_power_W() * 1.e9

    def get_power_pW(self):
        return self.get_power_W() * 1.e12

    def get_power_dbm(self):
        return self.watts_to_dbm(self.get_power_W())


class LaserTunable(Laser, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def set_wavelength_m(self, wavelength_m):
        '''Sets the wavelength in [m] of the laser.

        Args:
            wavelength_m (int, float): The wavelength in metres to
                set the laser to.

        Returns:
            float: The wavelength in metres the laser was set to.
        '''
        pass

    @abc.abstractmethod
    def get_wavelength_m(self):
        '''Gets the wavelength in [m] of the laser.

        Returns:
            float: The wavelength in metres the laser is set to.
        '''
        pass

    @abc.abstractmethod
    def get_velocity_nm_s(self):
        pass

    @abc.abstractmethod
    def set_velocity_nm_s(self, velocity):
        pass

    def set_wavelength_um(self, wavelength):
        return self.set_wavelength_m(wavelength * 1.e-3) * 1.e3

    def set_wavelength_um(self, wavelength):
        return self.set_wavelength_m(wavelength * 1.e-6) * 1.e6

    def set_wavelength_nm(self, wavelength):
        return self.set_wavelength_m(wavelength * 1.e-9) * 1.e9

    def get_wavelength_mm(self):
        return self.get_wavelength_m() * 1.e3

    def get_wavelength_um(self):
        return self.get_wavelength_m() * 1.e6

    def get_wavelength_nm(self):
        return self.get_wavelength_m() * 1.e9

    def get_wavelength_power_scan_manual(self, start_wavelength_nm, stop_wavelength_nm,
                                         step_wavelength_nm, power_meters=None,
                                         delay_wavelength_changes_s=1., filename=None):
        '''
        Performs a wavelength sweep.

        Manually changes the laser power and the measures.  Does not use the in-built sweep
        function like Thach's sweep does.

        Args:
            start_wavelength_nm (int, float): The minimum wavelength value to start the sweep.
            stop_wavelength_nm (int, float): The maximum wavelength value to stop the sweep.
            step_wavelength_nm (int, float): The wavelength stepsize.
            power_meters (PowerMeter,list,None): The constructed power meter object that should be
                read as the wavelength is swept.  If a list of `PowerMeter` is given, all power meters
                in the list will be read.  `None` if no power meter should be read.
            power_meter_units (str): 'W' if the power should be read and returned in Watts, or
                'dBm' if the power should be read and returned in 'dBm'.
            delay_wavelength_changes_s (int, float): The delay in seconds between laser wavelength
                changes.
            filename (None,str): The name of the file to save the wavelength and power data
                to.  The data is not saved to a file if `None`.

        Returns:
            list: The first item is a float list of the wavelengths swept.  Subsequent
                items in the list are power meter readings given in the same order as in
                `power_meters.`
        '''

        if filename:
            fs = open(filename, 'w')

        # Wavelengths to measure.
        wavelengths_nm = np.arange(start_wavelength_nm, stop_wavelength_nm, step_wavelength_nm)

        # Convert parameters of power_meters.
        try:
            power_meters.__iter__
        except AttributeError:
            power_meters = [power_meters]

        # Store initial settings.
        laser_wavelength_m = self.get_wavelength_m()
        power_meter_wavelength_m = [pm.get_wavelength_m() for pm in power_meters]

        # Set power meters to average wavelength in scan.
        for pm in power_meters:
            pm.set_wavelength_nm(0.5*(start_wavelength_nm+stop_wavelength_nm))

        # Turn laser on.
        self.turn_on()

        # Measure powers.
        p_store = []
        progress_bar = tqdm(wavelengths_nm)
        for w in progress_bar:
            self.set_wavelength_nm(w)
            time.sleep(delay_wavelength_changes_s)
            p = [pm.get_power_nW() for pm in power_meters]
            p_store.append(p)

            if filename:
                w_p = [w]
                w_p.extend(p)
                s = ','.join([str(v) for v in w_p])
                fs.write(s+'\n')

            progress_bar.set_description('Power: %0.3e' % p[0])

        # Turn laser off.
        self.turn_off()

        # Organise wavelengths and powers.
        p_store_T = zip(*p_store)
        r = [wavelengths_nm]
        r.extend(p_store_T)

        if filename:
            fs.close()

        # Restore initial settings.
        self.set_wavelength_m(laser_wavelength_m)
        for pm, wl in zip(power_meters, power_meter_wavelength_m):
            pm.set_wavelength_nm(wl)

        return r

    def wavelength_sweep_manual_interp(self,
                                       start_wavelength_nm,
                                       stop_wavelength_nm,
                                       step_wavelength_nm,
                                       power_meters,
                                       delay_wavelength_changes_s=1.,
                                       filename=None):
        wavelengths, powers = self.get_wavelength_power_scan_manual(
                                                    start_wavelength_nm,
                                                    stop_wavelength_nm,
                                                    step_wavelength_nm,
                                                    power_meters,
                                                    delay_wavelength_changes_s=1.,
                                                    filename=None)
        powers_interp = interpolate.interp1d(wavelengths, powers)
        return powers_interp



import abc
import time
import math

class PowerMeter(object, metaclass=abc.ABCMeta):
    '''
    Abstract base interface class for power meters.

    All power meter drivers should derive from this class
    and call `super.__init__(...)` in their constructor.
    '''
    def __init__(self):
        pass

    @abc.abstractmethod
    def get_analogue_current_A(self, analogue_voltage_V):
        '''
        Get the detector photocurrent [A].  This should be directly
        propertional to the detector analogue voltage [V] output.

        Args:
            analogue_voltage_V (float, np.array): Analogue voltage [V]
                representing the current.

        Returns:
            float, np.array: Analogue current [A] corresponding to
            the given analogue voltage [V].
        '''
        pass

    @abc.abstractmethod
    def get_responsivity_A_W(self):
        '''
        Returns the responsivity [A/W] of the detector.

        Returns:
            float: Responsivity [A/W] of the detector.
        '''
        pass

    @abc.abstractmethod
    def _get_power_W(self):
        '''
        Read the power meter.

        This function is not to be called.  One should prefer
        the not underscored getter functions which allow for
        averaging.

        Returns:
            float: The power in [W].
        '''
        pass

    @abc.abstractmethod
    def get_wavelength_m(self, wavelength_m):
        '''
        Gets the wavelength the power meter is set to read.

        Returns:
            float: The wavelength in [m] the power meter
            is operating in.
        '''
        pass

    @abc.abstractmethod
    def set_wavelength_m(self, wavelength_m):
        '''
        Sets the wavelength the power meter is set to read.

        Args:
            wavelength_m (int, float): The wavelength in
                [m] to set the power meter to.

        Returns:
            float: The wavelength the power meter is operating
                in.
        '''
        pass

    def set_wavelength_um(self, wavelength_um):
        return self.set_wavelength_m(wavelength_um * 1.e-6)

    def set_wavelength_nm(self, wavelength_nm):
        return self.set_wavelength_m(wavelength_nm * 1.e-9)

    def get_power_W(self, average=1, read_period_ms=250.):
        '''
        Read the power meter.

        Args:
            average (int): How many power readings should be
                averaged over before returning a power.
            read_period_ms (int, float): The time to wait between
                power readings.  Only makes sense if `average > 1`.

        Returns:
            float: The, perhaps averaged, power reading.
        '''
        powers = []
        if average > 1:
            for i in range(average):
                powers.append(self._get_power_W())
                time.sleep(read_period_ms / 1000.)
            avg_power = sum(powers) / float(len(powers))
        else:
            avg_power = self._get_power_W()
        return avg_power

    def get_power_mW(self, average=1, read_period_ms=None):
        return self.get_power_W(average, read_period_ms) * 1.e3

    def get_power_uW(self, average=1, read_period_ms=None):
        return self.get_power_W(average, read_period_ms) * 1.e6

    def get_power_nW(self, average=1, read_period_ms=None):
        return self.get_power_W(average, read_period_ms) * 1.e9

    def get_power_pW(self, average=1, read_period_ms=None):
        return self.get_power_W(average, read_period_ms) * 1.e12

    def get_power_dbm(self, average=1, read_period_ms=None):
        power_W = self.get_power_W()
        power_dBm = PowerMeter.watts_to_dbm(power_W)
        return power_dBm

    def get_wavelength_mm(self):
        return self.get_wavelength_m() * 1.e3

    def get_wavelength_um(self):
        return self.get_wavelength_m() * 1.e6

    def get_wavelength_nm(self):
        return self.get_wavelength_m() * 1.e9

    @staticmethod
    def dbm_to_watts(power_dbm):
        '''Converts [dBm] to [W].

        Args:
            power_dbm (int, float): Power in [dBm].

        Returns:
            float: Power in [W].
        '''
        power_W = 10.**(power_dbm/10.) / 1.e3
        return power_W

    @staticmethod
    def watts_to_dbm(power_W):
        '''Converts [W] to [dBm].

        Args:
            power_W (int, float): Power in [W].

        Returns:
            float: Power in [dBm].
        '''
        if power_W < 1.e-12:
            power_dbm = -1000.
        else:
            power_dbm = 10.*math.log10(power_W/1.e-3)
        return power_dbm

    def get_analogue_power_W(self, analogue_voltage_V):
        '''
        Calculates the power of the power meter based on the
        analogue voltage value read.  The analogue voltage
        should be directly related to the current.

        Args:
            analogue_voltage_V (float, np.array): Analogue voltage [V]
                representing the current.

        Returns:
            float, np.array: The power [W] corresponding to the
                given analogue voltage [V].
        '''
        curr_A = self.get_analogue_current_A(analogue_voltage_V)
        resp_A_W = self.get_responsivity_A_W()
        pow_W = curr_A / resp_A_W
        return pow_W



from . import laser as las
from .agilent_lightwave_connection import AgilentLightWaveConnection
from ..utils import gnuplot as gp
import time
import struct
import numpy as np
from scipy import interpolate
import os
import serial as ser
import warnings

class LaserAgilent8164B(AgilentLightWaveConnection, las.LaserTunable):
    '''
    Controls the laser module in the Agilent 8164B.

    Args:
        gpib_num (int): The GPIB bus number the laser is on.
        gpib_dev_num (int): The device number the laser in on
            the bus.
        power_unit (str): Either \'W\' or \'dBm\' depending on what
            units the Agilent 8164B should use for the laser power.
        output_mode (str):
            'HIGH' -> The High Power output is regulated.
            'LOWS' -> The Low SSE output is regulated.
            'BHR' -> Both outputs are active but only the High Power output is Regulated.
            'BLR' -> Both outputs are active but only the Low SSE output is Regulated.
    '''
    def __init__(self, gpib_num=None, gpib_dev_num=None, power_unit='W',
                 output_mode='high'):
        super().__init__(gpib_num, gpib_dev_num)
        self._power_unit = self.set_unit(power_unit)
        self._set_output_mode(output_mode)

    def turn_on(self):
        # Turn laser on.
        self._write('outp1')
        # Open output shutter.
        self._write('sour0:pow:stat 1')
        return self.get_on_or_off()

    def turn_off(self):
        # Turn laser off.
        self._write('outp0')
        # Open output shutter.
        self._write('sour0:pow:stat 0')
        return self.get_on_or_off()

    def get_on_or_off(self):
        on_off = self._query('sour0:pow:stat?')
        return bool(int(on_off))

    def set_power_W(self, power_W):
        assert power_W < 5.e-3, 'Can\'t set power that high.'
        if self._power_unit == 'dBm':
            power_dbm = LaserAgilent8164B.watts_to_dbm(power_W)
            cmd = 'sour0:pow %fdbm' % power_dbm
        elif self._power_unit == 'W':
            cmd = 'sour0:pow %fw' % power_W
        self._write(cmd)
        return self.get_power_W()

    def get_power_W(self):
        inString = 'sour0:pow?'
        data = self._query(inString)
        return float(data)

    def set_wavelength_m(self, wavelength_m):
        cmd = 'sour0:wav %sM' % wavelength_m
        self._write(cmd)
        r = self.get_wavelength_m()
        return r

    def get_wavelength_m(self):
        cmd = 'sour0:wav?'
        r = float(self._query(cmd))
        return r

    def set_unit(self, unit):
        '''
        Sets the units the laser should operate in.

        This will affect in what units the laser power
        is displayed on the screen.

        Args:
            unit (str): Either \'dBm\' or \'W\'.

        Returns:
            str: The units the laser is oeprating in.
        '''
        assert unit.lower() in ('dbm', 'w')
        cmd = 'sour0:pow:unit %s' % unit
        self._write(cmd)
        self._power_unit = self.get_unit()
        return self._power_unit

    def get_unit(self):
        '''
        Gets the units the laser is operating in.

        The units affect in what units the laser power
        is displayed on the screen.

        Returns:
            str: The units the laser is oeprating in.
        '''
        inString = 'sour0:pow:unit?'
        data = self._query(inString)
        unit = int(data)
        if unit == 0:
            unit_str = 'dBm'
        else:
            unit_str = 'W'
        return unit_str

    def last_operation_completed(self):
        return self._query('*OPC?')

    def wait_for_last_operation_completed(self):
        finish = False
        while not finish:
            finish = self.last_operation_completed()
        return finish

    def _get_output_mode(self):
        m = self._query('outp0:path?')
        return m

    def _set_output_mode(self, mode):
        '''
        Sets the mode the power meter will operate in.

        Args:
            mode (str):
                HIGH -> The High Power output is regulated.
                LOWS -> The Low SSE output is regulated.
                BHR -> Both outputs are active but only the High Power output is Regulated.
                BLR -> Both outputs are active but only the Low SSE output is Regulated.

        Returns:
            str: The mode the laser is operating in.
        '''
        m = mode.lower()
        assert m in ('high', 'lows', 'bhr', 'blr')
        self._write('outp0:path %s' % m)
        return self._get_output_mode()

    def get_velocity_nm_s(self):
        return float(self._query('sour0:wav:swe:spe?'))

    def set_velocity_nm_s(self, sweep_speed_nm_per_sec):
        assert float(sweep_speed_nm_per_sec) in (0.5, 1., 2., 5., 10., 20., 40., 80.), \
            'Invalid sweep speed choice; laser only supports certain sweep speeds nm/s.'
        self._write('sour0:wav:swe:spe %fnm/s' % sweep_speed_nm_per_sec)
        return self.get_velocity_nm_s()

    def wavelength_sweep(self,
                         start_wavelength_nm,
                         stop_wavelength_nm,
                         step_wavelength_nm,
                         max_power_mW,
                         sweep_speed_nm_per_sec=5.,
                         power_metre_slot=1,
                         int_time_ms=20.,
                         filename=None,
                         preserve_settings=True):
        assert step_wavelength_nm >= 0.6e-3, '`step_wavelength_nm` to small.'

        max_power_W = max_power_mW / 1000.
        int_time_us = int_time_ms * 1000.
        int_time_s = int_time_ms / 1000.

        # Save current settings
        if preserve_settings:
            cmds = ['trig:conf?',
                    'sour0:pow:stat?',
                    'trig0:outp?',
                    'trig0:inp?',
                    'sour0:wav:swe:star?',
                    'sour0:wav:swe:stop?',
                    'sour0:wav:swe:step?',
                    'sour0:wav:swe:mode?',
                    'sour0:wav:swe:llog?',
                    'trig1:inp?',
                    'sens'+str(power_metre_slot)+':pow:unit?',
                    'sens'+str(power_metre_slot)+':pow:rang:auto?',
                    'sens'+str(power_metre_slot)+':pow:rang?',
                    'sens'+str(power_metre_slot)+':pow:wav?',
                    'sens'+str(power_metre_slot)+':func:par:logg?']
            temps = [self._query(cmd) for cmd in cmds]

        max_power_dbm = LaserAgilent8164B.watts_to_dbm(max_power_W)

        # Setup lasers and triggers.
        self._write('trig:conf loop')
        self._write('sour0:pow:stat 1')
        self._query('sour0:pow:stat?')
        self._write('trig0:outp stf')
        self._write('trig0:inp sws')
        self._write('sour0:wav:swe:star %fnm' % start_wavelength_nm)
        self._write('sour0:wav:swe:stop %fnm' % stop_wavelength_nm)
        self._write('sour0:wav:swe:step %fnm' % step_wavelength_nm)
        self.set_velocity_nm_s(sweep_speed_nm_per_sec)
        self._write('sour0:wav:swe:mode cont')
        self._write('sour0:wav:swe:llog 1')

        param_check = self._query('sour0:wav:swe:chec?').strip()
        if param_check != '0,OK':
            raise RuntimeError('Invalid sweep parameters: `%s`.' % param_check)

        max_selectable_power_W = float(self._query('sour0:wav:swe:pmax? %fnm,%fnm' %
                                                  (start_wavelength_nm, stop_wavelength_nm)))
        if max_power_W > max_selectable_power_W:
            raise RuntimeError('Invalid `max_power` choice.')
        self._write('sour0:pow %fW' % max_power_W)

        # This command doesn't work for our laser module: int(self._query('sour0:wav:swe:exp?'))
        # so it must be manually calculated.
        expected_triggers = int(self._query('sour0:wav:swe:exp?'))
        power_meas_time = int_time_s*expected_triggers
        sweep_time = (stop_wavelength_nm-start_wavelength_nm)/sweep_speed_nm_per_sec
        if power_meas_time <= sweep_time:
            warnings.warn('The time needed for power readings %.2fsec is longer than the sweep %.2fsec.  Either reduce the integration time of the power meter, reduce the sweep speed, or reduce the sampling rate (`step_wavelength_nm`).' % (power_meas_time, sweep_time))
        self._write('sour0:wav:swe 1')

        # Setup power meter.
        self._write('trig1:inp sme')
        self._write('sens'+str(power_metre_slot)+':pow:unit 1')
        self._write('sens'+str(power_metre_slot)+':pow:rang:auto 0')
        self._write('sens'+str(power_metre_slot)+':pow:rang %fdbm' % max_power_dbm)
        self._query('sens'+str(power_metre_slot)+':pow:rang?')
        centre_wavelength_nm = 0.5*(start_wavelength_nm+stop_wavelength_nm)
        self._write('sens'+str(power_metre_slot)+':pow:wav %fnm' % centre_wavelength_nm)
        self._write('sens'+str(power_metre_slot)+':func:par:logg %i,%ius' % (expected_triggers, int_time_us))

        # Set to one trigger per measurment.
        self._write('sens'+str(power_metre_slot)+':func:stat logg,star')

        finished = False
        while not finished:
            flag = int(self._query('source0:wav:sweep:flag?').strip())
            finished = bool(flag)

        self._write('sour0:wav:swe:soft')

        # Wait for log to complete.
        while self._query('sour0:wav:sweep:state?').strip() != '+0':
            time.sleep(0.1)

        # Version from manual.  Doesn't work.
        #while self._query('sens'+str(power_metre_slot)+':func:stat?').strip() != 'LOGGING_STABILITY,COMPLETE':
        #    self._query('sens'+str(power_metre_slot)+':func:stat?')
        #    time.sleep(50.e-3)

        # Get logged data.
        self._write('sens'+str(power_metre_slot)+':func:res?')
        r = self._read(2)
        assert r[0] == '#', 'A \'#\' should have been read first.'
        size_of_num_bytes = int(r[1])
        num_bytes = int(self._read(size_of_num_bytes))
        data = self._read_raw(num_bytes)
        _ = self._read(1) # Read the remaining '\n' character.
        data = [data[i:i+4] for i in range(0, len(data), 4)]
        powers = [struct.unpack('f', d)[0] for d in data]

        while int(self._query('sour0:wav:swe:flag?')) not in (flag+1, flag+2):
            time.sleep(50.e-3)

        self._write('sour0:read:data? llog')
        r = self._read(2)
        assert r[0] == '#', 'A \'#\' should have been read first.'
        size_of_num_bytes = int(r[1])
        num_bytes = int(self._read(size_of_num_bytes))
        data = self._read_raw(num_bytes)
        _ = self._read(1) # Read the remaining '\n' character.
        data = [data[i:i+8] for i in range(0, len(data), 8)]
        wavelengths = [struct.unpack('d', d)[0] for d in data]

        # Wait for sweep to finish incase it is still running.
        while int(self._query('sour0:wav:swe?')) != 0:
            time.sleep(50.e-3)

        # Reload settings
        if preserve_settings:
            for cmd, temp in zip(cmds, temps):
                self._write(cmd[:-1] + ' ' + temp)

        # Save data to file if necessary.
        try:
            with open(filename, 'w') as fs:
                for w, p in zip(wavelengths, powers):
                    fs.write('%e,%e\n' % (w, p))
            script_dir = os.path.dirname(os.path.abspath(__file__))
            gp.Gnuplot(script_dir+'/agilent_laser_sweep.gpi', {'filename':filename})
        except TypeError:
            pass

        assert expected_triggers == len(wavelengths) == len(powers), 'Invalid data.'

        return wavelengths, powers

    def wavelength_sweep_interp(self,
                                start_wavelength_nm,
                                stop_wavelength_nm,
                                step_wavelength_nm,
                                max_power_W,
                                sweep_speed_nm_per_sec=5.,
                                filename=None):
        wavelengths, powers = self.wavelength_sweep(start_wavelength_nm,
                                                    stop_wavelength_nm,
                                                    step_wavelength_nm,
                                                    max_power_W,
                                                    sweep_speed_nm_per_sec,
                                                    filename)
        powers_interp = interpolate.interp1d(wavelengths, powers)
        return powers_interp



from setuptools import setup

setup(name='agilent-lightwave',
      version='0.2',
      description='Python driver for the Agilent Lightwave 8164A/B.',
      url='https://github.com/jtambasco/agilent-lightwave',
      author='Jean-Luc Tambasco',
      author_email='an.obscurity@gmail.com',
      license='MIT',
      install_requires=[
          'tqdm',
          'scipy',
          'numpy'
      ],
      packages=['agilent-lightwave'],
      include_package_data=True,
      zip_safe=False)
